# -*- coding: utf-8 -*-
import sys
from StringIO import StringIO
import unittest
from skoolkittest import SkoolKitTestCase
from skoolkit.skoolparser import SkoolParser
from skoolkit.skoolctl import CtlWriter

TEST_SKOOL = 'test-ctl.skool'

TEST_CTL = """c 32768 Routine
D 32768 Routine description
R 32768 A Some value
R 32768 B Another value
  32768,1 Do nothing
B 32769,2,2 1-line B sub-block
B 32771,3,1,2 2-line B sub-block
D 32774 Mid-block comment
T 32774,5,5 T sub-block
W 32779,2 W sub-block
Z 32781,2 Z sub-block
M 32783,12 Sub-block with instructions of various types
B 32785,1
W 32786,4,4
T 32790,2,2
Z 32792,3
E 32768 End comment
i 32796 Ignore block
b 49152 Data block
g 49153 Game status buffer entry
W 49153
t 49155 Message
  49155,2,2
u 49157 Unused block
w 49158 Word block
  49158,4,2
z 49162 Zero block
i 49172 Another ignore block""".split('\n')

class CtlWriterTest(SkoolKitTestCase):
    def _get_writer(self, elements, write_hex):
        case = base = None
        level = fix_mode = 0
        verbose = parser_warn = html = create_labels = False
        mode_args = (case, base, level, verbose, parser_warn, fix_mode, html, create_labels)
        parser = SkoolParser(TEST_SKOOL, mode_args)
        return CtlWriter(parser.memory_map, elements, write_hex)

    def _get_ctl(self, elements='btdrmsc', write_hex=False):
        writer = self._get_writer(elements, write_hex)
        out = StringIO()
        writer.write(out)
        return out.getvalue().split('\n')[:-1]

    def test_default_elements(self):
        ctl = self._get_ctl()
        self.assertEqual(len(ctl), len(TEST_CTL))
        for i, line in enumerate(ctl):
            self.assertEqual(line, TEST_CTL[i])

    def test_default_elements_hex(self):
        ctl = self._get_ctl(write_hex=True)
        self.assertEqual(len(ctl), len(TEST_CTL))
        for i, line in enumerate(ctl):
            hex_addr = '$%04X' % int(TEST_CTL[i][2:7])
            self.assertEqual(line[2:7], hex_addr)

    def test_wb(self):
        ctl = self._get_ctl('wb')
        self.assertEqual(ctl[0], TEST_CTL[0][:7])

    def test_wbt(self):
        ctl = self._get_ctl('bt')
        self.assertEqual(ctl[0], TEST_CTL[0])

    def test_wbd(self):
        ctl = self._get_ctl('bd')
        self.assertEqual(ctl[1], TEST_CTL[1])

    def test_wbr(self):
        ctl = self._get_ctl('br')
        self.assertEqual(ctl[1], TEST_CTL[2])
        self.assertEqual(ctl[2], TEST_CTL[3])

    def test_wbm(self):
        ctl = self._get_ctl('bm')
        self.assertEqual(ctl[1], TEST_CTL[7])

    def test_wbs(self):
        ctl = self._get_ctl('bs')
        self.assertEqual(ctl[1], TEST_CTL[5][:11])

    def test_wbsc(self):
        ctl = self._get_ctl('bsc')
        self.assertEqual(ctl[1], TEST_CTL[4])

if __name__ == '__main__':
    unittest.main()
