# -*- coding: utf-8 -*-
import sys
import random
import unittest
from skoolkittest import SkoolKitTestCase
from skoolkit.skoollib import SkoolGame
from skoolkit.skoolparser import SkoolParser, RefParser

TEST_SKOOL = 'test-lib.skool'
TEST_REF = 'test-lib.ref'

class MockSkoolParser:
    def __init__(self, snapshot, entries=(), memory_map=()):
        self.snapshot = snapshot
        self.entries = entries
        self.memory_map = memory_map
        self.skoolfile = ''

class SkoolGameTest(SkoolKitTestCase):
    def _get_game(self, use_skool=False, use_ref=False, snapshot=None):
        ref_parser = RefParser()
        if use_ref:
            ref_parser.parse(TEST_REF)
        if use_skool:
            case = base = None
            level = fix_mode = 0
            verbose = parser_warn = create_labels = False
            html = True
            mode_args = (case, base, level, verbose, parser_warn, fix_mode, html, create_labels)
            skool_parser = SkoolParser(TEST_SKOOL, mode_args)
        else:
            skool_parser = MockSkoolParser(snapshot)
        return SkoolGame(skool_parser, ref_parser)

    def _assert_scr_equal(self, game, x0=0, y0=0, w=32, h=24):
        snapshot = game.snapshot[:]
        scr = game.get_screenshot(x0, y0, w, h)
        self.assertEqual(len(scr),  min((h, 24 - y0)))
        self.assertEqual(len(scr[0]), min((w, 32 - x0)))
        for j, row in enumerate(scr):
            for i, udg in enumerate(row):
                x, y = x0 + i, y0 + j
                df_addr = 16384 + 2048 * (y // 8) + 32 * (y % 8) + x
                af_addr = 22528 + 32 * y + x
                self.assertEqual(udg.data, snapshot[df_addr:df_addr + 2048:256], 'Graphic data for cell at (%i,%i) is incorrect' % (x, y))
                self.assertEqual(udg.attr, snapshot[af_addr], 'Attribute byte for cell at (%i,%i) is incorrect' % (x, y))

    def test_get_screenshot(self):
        snapshot = [0] * 65536
        for a in range(16384, 23296):
            snapshot[a] = random.randint(0, 255)
        game = self._get_game(snapshot=snapshot)
        self._assert_scr_equal(game)
        self._assert_scr_equal(game, 1, 2, 12, 10)
        self._assert_scr_equal(game, 10, 10)

    def test_get_instruction_addr_str(self):
        game = self._get_game(use_skool=True)
        self.assertEqual(game.get_instruction_addr_str(24576), '24576')
        self.assertEqual(game.get_instruction_addr_str(24579), '6003')

    def test_ref_parsing(self):
        game = self._get_game(use_ref=True)
        self.assertEqual(game.info, {'X': 'foo', 'Y': 'bar'})

if __name__ == '__main__':
    unittest.main()
